/*
  ZipIntSFX.h - Header file of internal SFX unit for TZipBuilder
                Code is derived from TZipBuilderSFX component
  TZipBuilder VCL by Chris Vleghert and Eric W. Engler
  v1.79
  Copyright (C) 2005  Roger Aelbrecht

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License (licence.txt) for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: Zipbuilder AT drehoeksw DOT net
  DelphiZip maillist subscribe at http://www.geocities.com/SiliconValley/Orchard/8607/ 
  updates: http://www.drehoeksw.net

  last modified 2005-02-06
---------------------------------------------------------------------------*/
#ifndef ZipIntSFXH
#define ZipIntSFXH

#ifndef NO_SFX

#include "ZBStructs.h"
#include "CondDefines.h"
#include "Ziputils.h"


//#define ZipBuilderVersion180  // for latest version TZipBuilder
// this zipbuilder is not yet available

bool CALLBACK ResEnumFunc(HANDLE hMod, char* szType, char* szName, TStrings* lParam);

namespace Zipintsfx
{
//-- type declarations -------------------------------------------------------

enum SpanSfxType {stSpanSfx, stMVSfx};
enum TSFXOption { soAskCmdLine, soAskFiles, soHideOverWriteBox, soAutoRun, soNoSuccessMsg,
                  soExpandVariables, soInitiallyHideFiles, soForceHideFiles,
                  soCheckAutoRunFileName, soCanBeCancelled, soDetached };
enum TSFXOverwriteMode { somOverwrite, somSkip, somAsk };
typedef Set<TSFXOption, soAskCmdLine, soDetached>  TSFXOptions;
typedef Set<TSFXOption, soExpandVariables, soDetached> TSFXOptionsExt;


#pragma pack(push, 1)
struct TSFXFileHeader
{
	unsigned Signature;
	Word Size;
	TSFXOptions Options;
	TSFXOverwriteMode DefOVW;
	Byte CaptionSize;
	Byte PathSize;
	Byte CmdLineSize;
	Byte RegFailPathSize;
	Byte StartMsgSize;
	unsigned StartMsgType;
} ;

struct TSFXFileEndOfHeader
{
	unsigned Signature;
	unsigned HeaderSize;
} ;

struct TSFXDetachedHeader
{
  unsigned Signature;
  unsigned NameLen; // length of filename without extension of detached archive
  unsigned ExtLen;  // length of file ext without leading .
} ;

#pragma pack(pop)

#if __BORLANDC__ < 1344
#pragma option -a4
#else
#pragma option -a8
#endif
//-----------------------------------------------------------------
class PACKAGE TZipIntSFX : public Classes::TComponent
{

private:
  bool Detached;
  TComponent* FBuilder;
  AnsiString FMySFXMessage;
	unsigned FMySFXMessageFlags;
	AnsiString FRegFailPath;
	AnsiString FSFXLanguage;
	TSFXOptions FMySFXOptions;
  TSFXOverwriteMode FMySFXOverWriteMode;
  TSFXOptionsExt FSFXOptionsExt;
  AnsiString FSFXPath;
  bool FUseDelphiBin;
  TStringList* Included_Languages;
  AnsiString BinVersion;
  AnsiString ResVersion;
	TMemoryStream* SFXBinStream;
  DWord  __fastcall BrowseResDir( pIRD ResStart, pIRD dir, int Depth );
	int __fastcall CheckZipBuilder(void);
	int __fastcall CopyAndAdjustCentralDir(const int InFile, const int OutFile,
                                         const int NumEntries, const int DiskToAdjust,
                                         const int Offset);
	int __fastcall CopyBuffer(const int InFile, const int OutFile, int ReadLen);
	bool __fastcall CreateSFXBinStream(void);
	int __fastcall GetFirstLocalHeader(const int InFile, const int NumEntries,
                                     const int WhichDisk, const int Position);
#ifndef NO_SPAN
  AnsiString __fastcall GetLastVolume(const AnsiString& FileName,ZipEndOfCentral& EOC);
#endif
	AnsiString __fastcall GetSFXLanguage(void);
	AnsiString __fastcall GetVersion(void);
  bool __fastcall LoadFromResource(const AnsiString& SfxType);
  bool __fastcall LookForBinFile(const AnsiString& Sfxtype);
  DWord  __fastcall LookForDirs( const void *SectionData, const DWord SectionVirtualStart,
															const int SectionLen, pIDD directories );
  __int64 __fastcall ReadEOC(const int InFile, ZipEndOfCentral &EOC);
  int __fastcall ReplaceIcon(const int &ExeFile, const int MaxPos, Graphics::TIcon* NewIcon);
  DWord  __fastcall SearchResDirEntry(pIRD ResStart, pIRDirE entry, int Depth);
  void __fastcall SetRegFailPath(const AnsiString Value);
  void __fastcall SetSFXLanguage(const AnsiString Value);
  void __fastcall SetSFXPath(const AnsiString Value);
	void __fastcall SetVersion(const AnsiString Value);
protected:
	int __fastcall DoConvertOrCreateSFX(const AnsiString& File, const bool CreateNew);
	void __fastcall GetPropertiesFromZipBuilder(void);
	virtual void __fastcall Notification(Classes::TComponent* aComponent, Classes::TOperation Operation);
	int __fastcall ReadNewSFXProperties(const int InFile, const int MaxPos);
	int __fastcall ReadSFXIcon(const AnsiString& File);
	int __fastcall ReadSFXProperties(const int InFile, const int MaxPos);
public:
	__fastcall virtual TZipIntSFX(Classes::TComponent* aOwner);
	__fastcall virtual ~TZipIntSFX(void);
	int __fastcall CreateNewSFX(const AnsiString& File);
	int __fastcall ConvertToSFX(void);
#ifndef NO_SPAN
  int __fastcall ConvertToSpanSFX(AnsiString OutFilePath);
#endif
	int __fastcall ConvertToZip(void);
__published:
  __property AnsiString RegFailPath={read=FRegFailPath, write=SetRegFailPath};
  __property AnsiString SFXLanguage={read=GetSFXLanguage, write=SetSFXLanguage};
  __property TSFXOptionsExt SFXOptionsExt={read=FSFXOptionsExt, write=FSFXOptionsExt,nodefault};
  __property AnsiString SFXPath = {read=FSFXPath, write=SetSFXPath};
  __property bool UseDelphiBin = {read=FUseDelphiBin, write=FUseDelphiBin, default=false};
	__property AnsiString Version = {read=GetVersion, write=SetVersion};
	__property TComponent* ZipBuilder = {read=FBuilder, write=FBuilder};
};

#pragma option -a.
#if __BORLANDC__ < 1344		// BCB4=0x540(1344)
 //-- template instantiations -------------------------------------------------
 template class TSFXOptions;
 template class TSFXOptionsExt;
#endif
}	/* namespace Zipintsfx */
#if !defined(NO_IMPLICIT_NAMESPACE_USE)
using namespace Zipintsfx;
#endif

#endif   // no_sfx

#endif	// ZipIntSFX
